"""
pretty_status_list.py
Version 1.00
(C) [#OMEGA] - K2

IMPORTANT

This file is designed to be used with
iptocountry

Agreement:
 
By using this file you agree to the following terms:

This file is delivered 'as-is'. 
I will not take resposibilty for any damage resulting of the use or not-use of this file. 

As for sharing permissions:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.
"""

# ##### #
# Imports  #
# ##### #

# ES Libs
import es
import usermsg
import gamethread
import langlib
import playerlib

# Python Libs
from time import time
from os.path import exists

# Custom Libs
import iptocountry

# ####### #
# Addon Info #
# ####### #

info = es.AddonInfo()
info.name        = "Pretty Status List"
info.version     = "1.00"
info.author      = "[#OMEGA] - K2"
info.url         = "http://www.g4tc.net"
info.description = "Provides more detailed and ordered information about any connected player."
info.tags        = "pstatus status pretty status list playerlist userlist list"

# ###### #
# Global Data #
# ###### #

lang = langlib.Strings(es.getString('eventscripts_gamedir') + "/addons/eventscripts/pretty_status_list/pretty_status_list-strings.ini")

users = {}
game = es.getString('eventscripts_gamedir').replace('\\', '/').rpartition('/')[2].lower() 
if game not in ['cstrike','dod','tf','empires']:
	game = 'unsupported'

# ######## #
# Load & Unload #
# ######## #

def load():
	# Register Commands
	es.regclientcmd('pstatus','pretty_status_list/print_client_info','Shows information about the players')
	es.regcmd('pstatus','pretty_status_list/print_console_info','Shows information about the players')
	# Set Pub vars
	es.set('pstatus_version',info.version)
	es.flags('add','replicated','pstatus_version')
	es.flags('add','notify','pstatus_version')
	
def unload():
	# Register Commands
	es.unregclientcmd('pstatus')
	# Cleanup pub vars
	es.flags('remove','replicated','pstatus_version')
	es.flags('remove','notify','pstatus_version')
	es.set('g4tc_emp_info_version','pstatus_version')

# ########### #
# Main Script Functions #
# ########### #

# Events

def player_disconnect(ev):
	userid = ev['userid']
	if users.has_key(userid):
		del users[userid]

# Functions

def print_client_info():
	print_info(es.getcmduserid())

def print_console_info():
	print_info(0)
	
def print_info(cmduserid):
	kicked = False
	# Spam protection - possible to crash the server with this command
	if cmduserid:
		global users
		if not users.has_key(cmduserid):
			users[cmduserid] = time()
		else:
			if time() - users[cmduserid] <= 1:
				es.server.queuecmd('kickid %s "Spamming pstatus command."' % cmduserid)
				kicked = True
			else:
				users[cmduserid] = time()
	if not kicked:
		language = GetLanguage(cmduserid)
		# Parse player info
		ppl = es.createplayerlist()
		team_msgs = [[DoubleFill(lang('team0',{},language),70)],
		[DoubleFill(lang('team1',{},language),70)],
		[DoubleFill(lang('%s_team2' % game,{},language),70)],
		[DoubleFill(lang('%s_team3' % game,{},language),70)]]
		for userid in ppl:
			msg = ''
			msg += Fill(str(ppl[userid]['index']),6)
			msg += Fill(str(userid),7)
			steamid = ppl[userid]['steamid'].replace('STEAM_0:','')
			msg += Fill(steamid,12)
			
			ip = ppl[userid]['address'].split(':')[0]
			if ip == '0':
				msg += Fill('BOT',16)
				msg += Fill('BOT',8)
			else:
				msg += Fill(ip,16)
				msg += Fill(iptocountry.get_location_data(ip)['country_3'],8)
			msg += ppl[userid]['name']
			msg += '\n'
			team_msgs[ppl[userid]['teamid']] += [msg,]
		# Print it to client console.
		ConPrint(cmduserid,lang('playerlist',{},language) + '\n\n')
		delay = 0.05
		for msgs in team_msgs:
			if cmduserid:
				gamethread.delayed(delay,SendData,(cmduserid,msgs))
				delay += 0.05
			else:
				SendData(cmduserid,msgs)

def SendData(cmduserid,msgs):
	language = GetLanguage(cmduserid)
	infoline = '\nIndex Userid SteamID     IP              %s %s\n\n' % (lang('country',{},language),lang('name',{},language))
	ConPrint(cmduserid,msgs[0])
	ConPrint(cmduserid,infoline)
	for msg in msgs[1:]:
		ConPrint(cmduserid,msg)
	ConPrint(cmduserid,' \n')

def ConPrint(cmduserid,msg):
	if cmduserid:
		usermsg.echo(cmduserid,msg)
	else:
		es.log(msg.replace('\n',''))
	
def Fill(string,amount):
	"""
	Fills both sides of the string with a white space.
	"""
	for i in xrange(0,amount - len(string)):
		string += ' '
	return string

def DoubleFill(string,amount):
	"""
	Fills both sides of the string with a minus.
	"""
	chars = amount-len(string)
	for i in xrange(0,int(chars/2)):
		string += '-'
		string = '-' + string
	if chars >= 0 and chars % 2:
		string += '-'
	return string
	
def GetLanguage(userid):
	if userid:
		return playerlib.getPlayer(userid).get('lang')
	else:
		return es.getString('eventscripts_language')